package org.opengroup.osdu.register.provider.gcp.mappers.osm;

import com.google.cloud.datastore.Key;
import org.opengroup.osdu.core.gcp.osm.persistence.IdentityTranslator;
import org.opengroup.osdu.core.gcp.osm.translate.Instrumentation;
import org.opengroup.osdu.core.gcp.osm.translate.TypeMapper;
import org.opengroup.osdu.register.ddms.model.Ddms;
import org.opengroup.osdu.register.provider.gcp.action.model.ActionDto;
import org.opengroup.osdu.register.provider.gcp.subscriber.model.SubscriptionDto;
import org.springframework.boot.autoconfigure.condition.ConditionalOnProperty;
import org.springframework.context.annotation.Scope;
import org.springframework.stereotype.Component;

import java.util.Arrays;
import java.util.Collections;
import java.util.HashMap;

import static org.springframework.beans.factory.config.BeanDefinition.SCOPE_SINGLETON;

/**
 * All Entity classes used in translation should be registered here even if don't need custom settings.
 * Each class is represented as Instrumentation object. At least class objects and rules for Identity translation should be provided.oi
 */
@Component
@Scope(SCOPE_SINGLETON)
@ConditionalOnProperty(name = "osmDriver")
public class OsmTypeMapperImpl extends TypeMapper {

    public OsmTypeMapperImpl() {
        super(Arrays.asList(
                new Instrumentation<>(SubscriptionDto.class,
                        new HashMap<String, String>() {{
                            put("pushEndpoint", "endpoint");
                            put("createdOnEpoch", "createdOn");
                        }},
                        new HashMap<String, Class<?>>() {{
                            put("createdOnEpoch", com.google.cloud.Timestamp.class);
                        }},
                        new IdentityTranslator<>(
                                SubscriptionDto::getId,
                                (r, o) -> r.setId(((Key) o).getName())
                        ),
                        Collections.singletonList("id")
                ),
                new Instrumentation<>(ActionDto.class,
                        new HashMap<String, String>() {{
                            put("createdOnEpoch", "createdOn");
                        }},
                        new HashMap<String, Class<?>>() {{
                            put("createdOnEpoch", com.google.cloud.Timestamp.class);
                        }},
                        new IdentityTranslator<>(
                                ActionDto::getId,
                                (r, o) -> r.setId(((Key) o).getName())
                        ),
                        Collections.singletonList("id")
                ),
                new Instrumentation<>(Ddms.class,
                        new HashMap<String, String>() {{
                            put("createdDateTimeEpoch", "createdOn");
                        }},
                        new HashMap<String, Class<?>>() {{
                            put("createdDateTimeEpoch", com.google.cloud.Timestamp.class);
                        }},
                        new IdentityTranslator<>(
                                Ddms::getId,
                                (r, o) -> r.setId(((Key) o).getName())
                        ),
                        Collections.singletonList("id")
                )
        ));
    }


}